@@ -379,8 +379,8 @@ object_isolate <- function(img,
                           plot = FALSE,
                           ...)
     segmented <- img[coord[1]:coord[2],
-                       coord[3]:coord[4],
-                       1:3]
+                     coord[3]:coord[4],
+                     1:3]
     return(segmented)
   }
 }
@@ -509,6 +509,232 @@ object_split <- function(img,
   return(list_objects)
 }
 
+#' Export multiple objects from an image to multiple images
+#'
+#' Givin an image with multiple objects, `object_export()` will split the
+#' objects into a list of objects using [object_split()] and then export them to
+#' multiple images into the current working directory (or a subfolder). Batch
+#' processing is performed by declaring a file name pattern that matches the
+#' images within the working directory.
+#'
+#' @inheritParams object_split
+#' @inheritParams utils_image
+#' @inheritParams analyze_objects
+#'
+#' @param pattern A pattern of file name used to identify images to be
+#'   processed. For example, if `pattern = "im"` all images in the current
+#'   working directory that the name matches the pattern (e.g., img1.-,
+#'   image1.-, im2.-) will be imported and processed. Providing any number as
+#'   pattern (e.g., `pattern = "1"`) will select images that are named as 1.-,
+#'   2.-, and so on. An error will be returned if the pattern matches any file
+#'   that is not supported (e.g., img1.pdf).
+#'@param dir_original The directory containing the original images. Defaults to
+#'  `NULL`. It can be either a full path, e.g., `"C:/Desktop/imgs"`, or a
+#'  subfolder within the current working directory, e.g., `"/imgs"`.
+#' @param dir_processed Optional character string indicating a subfolder within the
+#'   current working directory to save the image(s). If the folder doesn't
+#'   exist, it will be created.
+#' @param format The format of image to be exported.
+#' @param squarize Squarizes the image before the exportation? If `TRUE`,
+#'   [image_square()] will be called internally.
+#' @return A `NULL` object.
+#' @export
+#'
+#' @examples
+#' if(interactive()){
+#' library(pliman)
+#' img <- image_pliman("la_leaves.jpg")
+#' object_export(img)
+#'
+#' }
+object_export <- function(img,
+                          pattern = NULL,
+                          dir_original = NULL,
+                          dir_processed = NULL,
+                          format = ".jpg",
+                          squarize = FALSE,
+                          index = "NB",
+                          lower_size = NULL,
+                          watershed = TRUE,
+                          invert = FALSE,
+                          fill_hull = FALSE,
+                          filter = 2,
+                          threshold = "Otsu",
+                          extension = NULL,
+                          tolerance = NULL,
+                          object_size = "medium",
+                          edge = 20,
+                          remove_bg = FALSE,
+                          parallel = FALSE,
+                          verbose = TRUE){
+  if(is.null(pattern)){
+    list_objects <- object_split(img = img,
+                                 index = index,
+                                 lower_size = lower_size,
+                                 watershed = watershed,
+                                 invert = invert,
+                                 fill_hull = fill_hull,
+                                 filter = filter,
+                                 threshold = threshold,
+                                 extension = extension,
+                                 tolerance = tolerance,
+                                 object_size = object_size,
+                                 edge = edge,
+                                 remove_bg = remove_bg,
+                                 plot = FALSE,
+                                 verbose = FALSE)
+
+    a <- lapply(seq_along(list_objects), function(i){
+      tmp <- list_objects[[i]]
+      if(isTRUE(squarize)){
+        tmp <- image_square(tmp,
+                            plot = FALSE,
+                            sample_left = 10,
+                            sample_top = 10,
+                            sample_right = 10,
+                            sample_bottom = 10)
+      }
+      image_export(tmp,
+                   name = paste0("obj_",
+                                 leading_zeros(as.numeric(names(list_objects[i])),
+                                               n = 4), format),
+                   subfolder = dir_processed)
+    })
+  } else{
+
+    if(is.null(dir_original)){
+      diretorio_original <- paste0("./")
+    } else{
+      diretorio_original <-
+        ifelse(grepl("[/\\]", dir_original),
+               dir_original,
+               paste0("./", dir_original))
+    }
+    if(is.null(dir_processed)){
+      diretorio_processada <- paste0("./")
+    } else{
+      diretorio_processada <-
+        ifelse(grepl("[/\\]", dir_processed),
+               dir_processed,
+               paste0("./", dir_processed))
+    }
+
+    if(pattern %in% c("0", "1", "2", "3", "4", "5", "6", "7", "8", "9")){
+      pattern <- "^[0-9].*$"
+    }
+    plants <- list.files(pattern = pattern, diretorio_original)
+    extensions <- as.character(sapply(plants, file_extension))
+    names_plant <- as.character(sapply(plants, file_name))
+    if(length(grep(pattern, names_plant)) == 0){
+      stop(paste("Pattern '", pattern, "' not found in '",
+                 paste(getwd(), sub(".", "", diretorio_original), sep = ""), "'", sep = ""),
+           call. = FALSE)
+    }
+    if(!all(extensions %in% c("png", "jpeg", "jpg", "tiff", "PNG", "JPEG", "JPG", "TIFF"))){
+      stop("Allowed extensions are .png, .jpeg, .jpg, .tiff")
+    }
+
+    if(isTRUE(parallel)){
+
+      init_time <- Sys.time()
+      nworkers <- trunc(detectCores()*.3)
+      cl <- parallel::makePSOCKcluster(nworkers)
+      doParallel::registerDoParallel(cl)
+      on.exit(stopCluster(cl))
+
+      if(verbose == TRUE){
+        message("Processing ", length(names_plant), " images in multiple sessions (",nworkers, "). Please, wait.")
+      }
+      ## declare alias for dopar command
+      `%dopar%` <- foreach::`%dopar%`
+
+      results <-
+        foreach::foreach(i = seq_along(plants), .packages = c("pliman")) %dopar%{
+
+          tmpimg <- image_import(plants[[i]], path = diretorio_original)
+
+          list_objects <- object_split(img = tmpimg,
+                                       index = index,
+                                       lower_size = lower_size,
+                                       watershed = watershed,
+                                       invert = invert,
+                                       fill_hull = fill_hull,
+                                       filter = filter,
+                                       threshold = threshold,
+                                       extension = extension,
+                                       tolerance = tolerance,
+                                       object_size = object_size,
+                                       edge = edge,
+                                       remove_bg = remove_bg,
+                                       verbose = FALSE,
+                                       plot = FALSE)
+
+          a <- lapply(seq_along(list_objects), function(j){
+            tmp <- list_objects[[j]]
+            if(isTRUE(squarize)){
+              tmp <- image_square(tmp,
+                                  plot = FALSE,
+                                  sample_left = 10,
+                                  sample_top = 10,
+                                  sample_right = 10,
+                                  sample_bottom = 10)
+            }
+            image_export(tmp,
+                         name = paste0(file_name(plants[[i]]), "_",
+                                       leading_zeros(as.numeric(names(list_objects[j])),
+                                                     n = 4), format),
+                         subfolder = diretorio_processada)
+          }
+          )
+        }
+
+      message("Done!")
+      message("Elapsed time: ", sec_to_hms(as.numeric(difftime(Sys.time(),  init_time, units = "secs"))))
+
+    } else{
+
+      for(i in seq_along(plants)){
+        tmpimg <- image_import(plants[[i]], path = diretorio_original)
+
+        list_objects <- object_split(img = tmpimg,
+                                     index = index,
+                                     lower_size = lower_size,
+                                     watershed = watershed,
+                                     invert = invert,
+                                     fill_hull = fill_hull,
+                                     filter = filter,
+                                     threshold = threshold,
+                                     extension = extension,
+                                     tolerance = tolerance,
+                                     object_size = object_size,
+                                     edge = edge,
+                                     remove_bg = remove_bg,
+                                     verbose = FALSE,
+                                     plot = FALSE)
+
+        a <- lapply(seq_along(list_objects), function(j){
+          tmp <- list_objects[[j]]
+          if(isTRUE(squarize)){
+            tmp <- image_square(tmp,
+                                plot = FALSE,
+                                sample_left = 10,
+                                sample_top = 10,
+                                sample_right = 10,
+                                sample_bottom = 10)
+          }
+          image_export(tmp,
+                       name = paste0(file_name(plants[[i]]), "_",
+                                     leading_zeros(as.numeric(names(list_objects[j])),
+                                                   n = 4), format),
+                       subfolder = diretorio_processada)
+        }
+        )
+      }
+    }
+
+  }
+}
+
 
 
 #' Extract red, green and blue values from objects
