@@ -35,12 +35,11 @@
 NULL
 
 .onAttach <- function(libname, pkgname) {
-  vers <-  "2.0.0"
+  vers <-  "2.0.1"
   packageStartupMessage("|==========================================================|")
   packageStartupMessage("| Tools for Plant Image Analysis (pliman ", vers,  ")            |")
   packageStartupMessage("| Author: Tiago Olivoto                                    |")
-  packageStartupMessage("| Type 'citation('pliman')' to know how to cite pliman     |")
-  packageStartupMessage("| Type 'vignette('pliman_start')' for a short tutorial     |")
+  packageStartupMessage("| Type `citation('pliman')` to know how to cite pliman     |")
   packageStartupMessage("| Visit 'http://bit.ly/pkg_pliman' for a complete tutorial |")
   packageStartupMessage("|==========================================================|")
   check_ebi()
